/*
 * Copyright (C) MX4J.
 * All rights reserved.
 *
 * This software is distributed under the terms of the MX4J License version 1.0.
 * See the terms of the MX4J License in the documentation provided with this software.
 */

package mx4j.tools.adaptor.rmi;

import mx4j.MBeanDescriptionAdapter;

import java.lang.reflect.Method;
import java.lang.reflect.Constructor;

/**
 * Management interface descriptions for the RMIAdaptor MBean.
 * @author <a href="mailto:biorn_steedom@users.sourceforge.net">Simone Bordet</a>
 * @version $Revision: 1.1 $
 */
public class RMIAdaptorMBeanDescription extends MBeanDescriptionAdapter
{
   public String getAttributeDescription(String attribute)
   {
      if (attribute.equals("Running"))
      {
         return "The running status of this adaptor";
      }
      if (attribute.equals("Protocol"))
      {
         return "The underlying protocol used by this adaptor";
      }
      if (attribute.equals("JNDIName"))
      {
         return "The JNDI name under which a stub for the RMIConnector is registered";
      }
      if (attribute.equals("JNDIProperties"))
      {
         return "The JNDI properties";
      }
      if (attribute.equals("NamingProperties"))
      {
         return "The JNDI properties";
      }
      if (attribute.equals("HostName"))
      {
         return "The name of the host where this adaptor is running";
      }
      if (attribute.equals("HostAddress"))
      {
         return "The IP address of the host where this adaptor is running";
      }
      return super.getAttributeDescription(attribute);
   }

   public String getOperationDescription(Method operation)
   {
      String name = operation.getName();
      if (name.equals("start"))
      {
         return "Starts the adaptor";
      }
      if (name.equals("stop"))
      {
         return "Stops the adaptor";
      }
      if (name.equals("putJNDIProperty"))
      {
         return "Puts a property-value pair to be used by JNDI to create the InitialContext";
      }
      if (name.equals("putNamingProperty"))
      {
         return "Puts a property-value pair to be used by JNDI to create the InitialContext";
      }
      if (name.equals("clearJNDIProperties"))
      {
         return "Clears all previously inserted JNDI properties";
      }
      if (name.equals("clearNamingProperties"))
      {
         return "Clears all previously inserted JNDI properties";
      }
      return super.getOperationDescription(operation);
   }

   public String getOperationParameterName(Method method, int index)
   {
      String name = method.getName();
      if (name.equals("putJNDIProperty"))
      {
         switch (index)
         {
            case 0:
               return "property";
            case 1:
               return "value";
         }
      }
      if (name.equals("putNamingProperty"))
      {
         switch (index)
         {
            case 0:
               return "property";
            case 1:
               return "value";
         }
      }
      return super.getOperationParameterName(method, index);
   }

   public String getOperationParameterDescription(Method method, int index)
   {
      String name = method.getName();
      if (name.equals("putJNDIProperty"))
      {
         switch (index)
         {
            case 0:
               return "The property name";
            case 1:
               return "The property value";
         }
      }
      if (name.equals("putNamingProperty"))
      {
         switch (index)
         {
            case 0:
               return "The property name";
            case 1:
               return "The property value";
         }
      }
      return super.getOperationParameterDescription(method, index);
   }
}
